#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2009 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2009 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import time, os

from PyQt4.QtCore import QObject, SIGNAL, QVariant, QSettings
from pygoo import MemoryObjectGraph, ontology

from smewt.base import Media, Metadata
from smewt.base import ImportTask, SubtitleTask
from smewt.base.utils import GlobDirectoryWalker
from smewt.taggers import EpisodeTagger, MovieTagger
import cPickle as pickle

log = logging.getLogger('smewt.base.localcollection')

class LocalCollection(MemoryObjectGraph):
    '''This class represents a collection of resources in a local machine.  The collection
    is specified by a set of folders.
    This collection also keeps a last scanned datetime for each of the folders.
    '''

    def __init__(self, taskManager, settingsFile):
        super(LocalCollection, self).__init__()

        self.taskManager = taskManager
        self.settingsFile = settingsFile

        self.seriesFolders = {}
        self.seriesRecursive = True
        self.seriesFolderTimes = {}

        self.moviesFolders = {}
        self.moviesRecursive = True
        self.moviesFolderTimes = {}

        self.loadSettings()


    def loadSettings(self):
        try:
            self.seriesFolders, self.seriesRecursive, self.moviesFolders, self.moviesRecursive = pickle.load(open(self.settingsFile))
        except IOError, e:
            log.warning(e)

    def saveSettings(self):
        pickle.dump((self.seriesFolders, self.seriesRecursive, self.moviesFolders, self.moviesRecursive), open(self.settingsFile, 'w'))

    def getMoviesSettings(self):
        return self.moviesFolders.keys(), self.moviesRecursive

    def setMoviesFolders(self, folders, recursive):
        newfolders = {}
        for folder in folders:
            # try to keep the last scanned time (if any)
            newfolders[folder] = self.moviesFolders[folder] if folder in self.moviesFolders else None
        self.moviesFolders = newfolders
        self.moviesRecursive = recursive
        self.saveSettings()
        self.update()

    def getSeriesSettings(self):
        return self.seriesFolders.keys(), self.seriesRecursive

    def setSeriesFolders(self, folders, recursive):
        newfolders = {}
        for folder in folders:
            # try to keep the last scanned time (if any)
            newfolders[folder] = self.seriesFolders[folder] if folder in self.seriesFolders else None
        self.seriesFolders = newfolders
        self.seriesRecursive = recursive
        self.saveSettings()
        self.update()

    def add_object(self, *args, **kwargs):
        obj = args[0]
        if isinstance(obj, Media):
            self.updateLastScannedTimes([ obj ])
        return super(LocalCollection, self).add_object(*args, **kwargs)

    def removeNotPresent(self):
        # Remove the nodes that are not in one of the folders anymore
        def mediasOfUnselectedFolders(media, folders, recursive):
            for folder in folders:
                if recursive:
                    if os.path.abspath(media.filename).startswith(folder):
                        return False
                else:
                    if os.path.basename(os.path.abspath(media.filename)) == os.path.basename(folder):
                        return False

            return True


        mediasNotInSeries = self.find_all(node_type = Media,
                                         select = lambda x: mediasOfUnselectedFolders(x, self.seriesFolders.keys(), self.seriesRecursive))

        mediasNotInMovies = self.find_all(node_type = Media,
                                         select = lambda x: mediasOfUnselectedFolders(x, self.moviesFolders.keys(), self.moviesRecursive))

        # FIXME: implement me
        #self.nodes -= (set(mediasNotInSeries) & set(mediasNotInMovies))

    def modifiedFolders(self, folder, lastScanned, recursive):
        """Return the folders that have been modified since lastScanned."""
        result = []

        if lastScanned is None or lastScanned < os.path.getmtime(folder):
            result.append(folder)

        elif recursive:
            for f in GlobDirectoryWalker(folder, recursive = recursive):
                if os.path.isdir(f) and lastScanned < os.path.getmtime(f):
                    result.append(f)

        return result


    def reimportFolders(self, rescan):
        # Import those folders whose modified time
        # is larger than the last scan time
        for folder, lastScanned in self.seriesFolders.items():
            # It's possible that
            # it is a removeable drive
            if not os.path.isdir(folder):
                continue

            if rescan:
                self.importSeriesFolder(folder)
            else:
                modifiedFolders = self.modifiedFolders(folder, lastScanned, self.seriesRecursive)

                for modifiedFolder in modifiedFolders:
                    self.importSeriesFolder(modifiedFolder)

        # Import those folders whose modified time
        # is larger than the last scan time
        for folder, lastScanned in self.moviesFolders.items():
            # It's very possible that it is a removeable drive
            if not os.path.isdir(folder):
                continue

            if rescan:
                self.importMoviesFolder(folder)
            else:
                modifiedFolders = self.modifiedFolders(folder, lastScanned, self.moviesRecursive)

                for modifiedFolder in modifiedFolders:
                    self.importMoviesFolder(modifiedFolder)

    def rescan(self):
        # Remove those media that are not in one of the selected folders
        self.removeNotPresent()

        # Reimport all the folders
        self.reimportFolders(rescan = True)

    def update(self):
        # Remove those media that are not in one of the selected folders
        self.removeNotPresent()

        # Reimport only the folders that have changed
        self.reimportFolders(rescan = False)

    def importSeriesFolder(self, folder):
        self.seriesFolderTimes[folder] = time.mktime(time.localtime())
        self.importMediaFolder(folder, EpisodeTagger, self.seriesRecursive)

    def importMoviesFolder(self, folder):
        self.moviesFolderTimes[folder] = time.mktime(time.localtime())
        self.importMediaFolder(folder, MovieTagger, self.moviesRecursive)

    def importMediaFolder(self, folder, taggerType, recursive):
        filetypes = [ '*.avi',  '*.ogm',  '*.mkv', '*.sub', '*.srt' ]

        for filename in GlobDirectoryWalker(folder, filetypes, recursive):
            importTask = ImportTask(self, taggerType, filename)
            self.taskManager.add(importTask)

    def updateLastScannedTimes(self, mediaList):
        """Given a list of media that just got inserted into this collection, update the timestamps
        on the containing folders."""
        modified = False
        ontology.import_classes([ 'Movie', 'Series', 'Episode', 'Subtitle' ])

        for media in mediaList:
            if (media.metadata.node.isinstance(Movie) or
                (media.metadata.node.isinstance(Subtitle) and media.metadata.metadata.node.isinstance(Movie))):
                mediaFolders = self.moviesFolders
                folderTimes = self.moviesFolderTimes
            elif (media.metadata.node.isinstance(Episode) or
                  (media.metadata.node.isinstance(Subtitle) and media.metadata.metadata.node.isinstance(Episode))):
                mediaFolders = self.seriesFolders
                folderTimes = self.seriesFolderTimes
            else:
                log.warning("Can't update folder times for media: %s" % media)
                return

            # Since the media may have been of a subfolder we must find
            # the selected folder to which it belonged
            selectedFolder = [ folder for folder in mediaFolders if media.filename.startswith(folder) ]
            selectedFolder = sorted(selectedFolder, key = lambda x: len(x))[0] # sort by path length to get the deepest subdirectory

            # Set the last time the folder was scanned
            try:
                if mediaFolders[selectedFolder] != folderTimes[selectedFolder]:
                    mediaFolders[selectedFolder] = folderTimes[selectedFolder]
                    modified = True

            except KeyError:
                mediaFolders[selectedFolder] = time.mktime(time.localtime())
                modified = True

        if modified:
            self.saveSettings()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import cachedmethod, utils, SmewtException, Media
from smewt.guessers.guesser import Guesser
from smewt.media import Episode, Series, Movie
from smewt.base import textutils
from smewt.base.utils import smewtDirectory, smewtUserDirectory
from pygoo import MemoryObjectGraph

from PyQt4.QtCore import SIGNAL, QObject, QUrl

import os, sys, re, logging
from urllib import urlopen,  urlencode
from smewt.base.utils import curlget
from lxml import etree
import imdb

log = logging.getLogger('smewt.guessers.imdbmetadataprovider')


class Getter:
    def __init__(self, md, d):
        self.md = md
        self.d = d

    def get(self, name, fromName = None):
        if not fromName:
            fromName = name
        try:
            self.md.set(name, self.d[fromName])
        except KeyError:
            pass

    def getMultiUnicode(self, name):
        try:
            self.md.set(name, [ unicode(s) for s in self.d[name] ])
        except KeyError:
            pass


class IMDBMetadataProvider(object):
    def __init__(self):
        super(IMDBMetadataProvider, self).__init__()

        self.imdb = imdb.IMDb()

    @cachedmethod
    def getSeries(self, name):
        """Get the IMDBPy series object given its name."""
        results = self.imdb.search_movie(name)
        for r in results:
            if r['kind'] == 'tv series' or r['kind'] == 'tv mini series':
                return r
        raise SmewtException("EpisodeIMDB: Could not find series '%s'" % name)


    @cachedmethod
    def getEpisodes(self, series):
        """From a given IMDBPy series object, return a graph containing its information
        as well as its episodes nodes."""
        self.imdb.update(series, 'episodes')

        # TODO: debug to see if this is the correct way to access the series' title
        result = MemoryObjectGraph()
        smewtSeries = result.Series(title = series['title'])

        # FIXME: find a better way to know whether there are episodes or not
        try:
            series['episodes']
        except:
            return result

        for season in series['episodes']:
            for epNumber, episode in series['episodes'][season].items():
                try:
                    ep = result.Episode(series = smewtSeries,
                                        season = season,
                                        episodeNumber = epNumber)
                except:
                    # episode could not be entirely identified, what to do?
                    # can happen with 'unaired pilot', for instance, which has episodeNumber = 'unknown'
                    log.warning('invalid season/epnumber pair: %s / %s' % (season, epNumber))
                    continue # just ignore this episode for now

                g = Getter(ep, episode)
                g.get('title')
                g.get('synopsis', 'plot')
                g.get('originalAirDate', 'original air date')

        return result

    @cachedmethod
    def getMovie(self, name):
        """Get the IMDBPy movie object given its name."""
        log.debug('MovieIMDB: looking for movie %s', name)
        results = self.imdb.search_movie(name)
        for r in results:
            if r['kind'] == 'movie' or r['kind'] == 'video movie':
                return r
        raise SmewtException("MovieIMDB: Could not find movie '%s'" % name)

    @cachedmethod
    def getMovieData(self, movieImdb):
        """From a given IMDBPy movie object, return a graph containing its information."""
        self.imdb.update(movieImdb)
        result = MemoryObjectGraph()
        movie = result.Movie(title = movieImdb['title'],
                             year = movieImdb['year'])

        g = Getter(movie, movieImdb)
        g.get('rating')
        g.get('plot')
        g.get('plotOutline', 'plot outline')
        g.getMultiUnicode('writer') # documentaries don't have writers...
        g.getMultiUnicode('director')
        g.getMultiUnicode('genres')

        try:
            movie.cast = [ unicode(p) + ' -- ' + unicode(p.currentRole) for p in movieImdb['cast'][:15] ]
        except KeyError:
            movie.cast = []

        return result


    @cachedmethod
    def getPoster(self, imdbID):
        """Return the low- and high-resolution posters (if available) of an imdb object."""
        imageDir = smewtUserDirectory('images')
        noposter = smewtDirectory('smewt', 'media', 'common', 'images', 'noposter.png')

        loresFilename, hiresFilename = None, None

        try:
            movieUrl = 'http://www.imdb.com/title/tt' + imdbID
            html = etree.HTML(curlget(movieUrl))
            poster = html.find(".//div[@class='photo']")
            loresURL = poster.find('.//img').get('src')
            loresFilename = os.path.join(imageDir, '%s_lores.jpg' % imdbID)
            open(loresFilename, 'wb').write(curlget(loresURL))
        except SmewtException:
            log.warning('Could not find poster for imdb ID %s' % imdbID)
            return (noposter, noposter)

        try:
            hiresLink = poster.find('.//a')
            if hiresLink.get('title') == 'Poster Not Submitted':
                raise SmewtException('Poster not available')
            hiresHtmlURL = 'http://www.imdb.com' + hiresLink.get('href')
            html = etree.HTML(curlget(hiresHtmlURL))
            hiresURL = html.find(".//div[@class='primary']").find('.//img').get('src')
            hiresFilename = os.path.join(imageDir, '%s_hires.jpg' % imdbID)
            open(hiresFilename, 'wb').write(curlget(hiresURL))
        except SmewtException:
            log.warning('Could not find hires poster for imdb ID %s' % imdbID)
            hiresFilename = noposter


        return (loresFilename, hiresFilename)


    def startEpisode(self, episode):
        if episode.get('series') is None:
            raise SmewtException("IMDBMetadataProvider: Episode doesn't contain 'series' field: %s", md)

        name = episode.series.title
        try:
            series = self.getSeries(name)
            from smewt.base import cache
            #cache.save('/tmp/smewt.cache')
            eps = self.getEpisodes(series)
            #cache.save('/tmp/smewt.cache')

            lores, hires = self.getPoster(series.movieID)

            eps.find_one(Series).update({ 'loresImage': lores,
                                         'hiresImage': hires })

            return eps

        except Exception, e:
            log.warning(str(e) + ' -- ' + str(textutils.toUtf8(episode)))
            return MemoryObjectGraph()

    def startMovie(self, movieName):
        try:
            movieImdb = self.getMovie(movieName)
            result = self.getMovieData(movieImdb)

            movie = result.find_one('Movie')
            lores, hires = self.getPoster(movieImdb.movieID)
            movie.loresImage = lores
            movie.hiresImage = hires

            #result.displayGraph()
            return result

        except SmewtException, e:
            raise
            log.warning(str(e) + ' -- ' + textutils.toUtf8(movieName))
            return MemoryObjectGraph()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#



# filename- and network-related functions
import sys, os, os.path, fnmatch,  errno
import pycurl
from PyQt4.QtCore import QSettings, QVariant
import smewt
from pygoo.utils import tolist, toresult

class MethodID(object):
    def __init__(self, filename, module, className, methodName):
        self.filename = filename
        self.module = module
        self.className = className
        self.methodName = methodName

    def __str__(self):
        return 'module: %s - class: %s - func: %s' % (self.module, self.className, self.methodName)

def callerid():
    f = sys._getframe(1)

    filename = f.f_code.co_filename
    module = ''
    className = ''

    try:
        module = f.f_locals['self'].__class__.__module__
        className = f.f_locals['self'].__class__.__name__
    except:
        pass

    methodName = f.f_code.co_name

    return MethodID(filename, module, className, methodName)

def currentPath():
    '''Returns the path in which the calling file is located.'''
    return os.path.dirname(os.path.join(os.getcwd(), sys._getframe(1).f_globals['__file__']))

def makedir(path):
    try:
        os.makedirs(path)
    except OSError, e:
        if e.errno == errno.EEXIST:
            pass
        else: raise

def pathToUrl(path):
    if sys.platform == 'win32':
        # perform some drive letter trickery
        return 'localhost/%s:/%s' % (path[0], path[3:])

    return path

def smewtDirectory(*args):
    return os.path.join(currentPath(), '..', '..', *args)

def smewtDirectoryUrl(*args):
    return pathToUrl(smewtDirectory(*args))

def smewtUserDirectory(*args):
    settings = QSettings()
    userdir = unicode(settings.value('user_dir').toString())
    if not userdir:
        if sys.platform == 'win32':
            userdir = os.path.join(os.environ['USERPROFILE'], 'Application Data', 'Smewt')
        else:
            userdir = os.path.dirname(unicode(settings.fileName()))

        settings.setValue('user_dir',  QVariant(userdir))

    userdir = os.path.join(userdir, *args)
    makedir(userdir) # make sure directory exists

    return userdir

def smewtUserDirectoryUrl(*args):
    return pathToUrl(smewtUserDirectory(*args))


def splitFilename(filename):
    root, path = os.path.split(filename)
    result = [ path ]
    while path:
        root, path = os.path.split(root)
        result.append(unicode(path))
    return result


def guessCountryCode(filename):
    '''Given a subtitle filename, tries to guess which languages it contains.
    As a subtitle file can contain multiple subtitles, this function returns a list
    of found languages.'''

    # try to guess language from filename
    langs = [ lang.lower() for lang in filename.split('.') ]

    languages = { 'english': 'en',
                  'french': 'fr',
                  'francais': 'fr',
                  u'fran\xe7ais': 'fr',
                  'spanish': 'es',
                  'espanol': 'es',
                  u'espa\xf1ol': 'es'
                  }

    if len(langs) >= 3:
        l = langs[-2].lower()
        for lang, code in languages.items():
            if l[-2:] == lang[:2] or l[-3:] == lang[:3] or l[-len(lang):] == lang:
                return [ code ]

    # try to look inside the .idx, if it exists
    langs = set()
    if os.path.exists(filename[:-3] + 'idx'):
        lines = open(filename[:-3] + 'idx').readlines()
        for l in lines:
            if l[:3] == 'id:':
                langs.add(l[4:6])

    if langs:
        return list(langs)

    return [ 'unknown' ]

class GlobDirectoryWalker:
    # a forward iterator that traverses a directory tree

    def __init__(self, directory, patterns = ['*'], recursive = True):
        self.stack = [directory]
        self.patterns = patterns
        self.files = []
        self.index = 0
        self.recursive = recursive

    def __getitem__(self, index):
        while True:
            try:
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # pop next directory from stack
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # got a filename
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and self.recursive:
                    self.stack.append(fullname)
                for pattern in self.patterns:
                    if fnmatch.fnmatch(file, pattern):
                        return fullname


class CurlDownloader:
    def __init__(self):
        self.contents = []
        self.c = pycurl.Curl()

    def callback(self, buf):
        self.contents.append(buf)

    def get(self, url):
        self.contents = []
        c = self.c
        c.setopt(c.URL, url)
        c.setopt(c.WRITEFUNCTION, self.callback)
        c.setopt(c.COOKIEFILE, '')
        c.setopt(c.FOLLOWLOCATION, 1)
        c.perform()

        self.contents = ''.join(self.contents)

        return self.contents

    def __del__(self):
        self.c.close()

def curlget(url):
    c = CurlDownloader()
    return c.get(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import with_statement
from smewtexception import SmewtException
from Queue import Queue
from threading import Thread, Lock
import time
import logging

log = logging.getLogger('smewt.base.taskmanager')

class Task(object):
    def __init__(self, priority = 5):
        self.priority = priority

    def perform(self):
        """All tasks should implement this function, which should perform the actual task.

        No data is passed to this method. All the necessary task data should be given in the constructor.

        This method shouldn't return anything on success, and raise an exception in case of failure.
        It can have side effects though, such as updating the global collection for instance. In this case,
        the collection needs to be passed as argument to the constructor."""
        raise NotImplementedError


def worker(queue):
    while True:
        try:
            (_, _), task = queue.get()
            # TODO: need to have timeout on the tasks, eg: 5 seconds
            task.perform()
        except SmewtException, e:
            log.warning('TaskManager: task failed with error: %s' % e)
        except TypeError, e:
            # to avoid errors when the program exits and the queue has been deleted
            # FIXME: catches way too much, we need to somehow tell the thread to stop when the TaskManager gets deleted
            log.warning('TaskManager: task failed with type error: %s' % e)
            raise
            pass
        except Exception, e:
            log.warning('TaskManager: task failed unexpectedly with error: %s' % e)
        finally:
            queue.task_done()


# FIXME: switch me to PriorityQueue when we have python2.6
class TaskManager(Queue, object):
    """The TaskManager is a stable priority queue of tasks. It takes them one by one, the
    one with the highest priority first, and call its perform() method, then repeat until no tasks are left.

    If two or more tasks have the same priority, it will take the one that was added first to the queue.

    The TaskManager can be controlled asynchronously, as it runs the tasks in a separate thread."""

    def __init__(self, progressCallback = None):
        super(TaskManager, self).__init__()

        self.progressCallback = progressCallback

        self.total = 0
        self.totalLock = Lock()

        t = Thread(target = worker, args = (self,))
        # FIXME: for python2.6
        #t.daemon = True
        t.setDaemon(True)
        t.start()

    def add(self, task):
        # -task.priority because it always gets the lowest one first
        # we need to put the time as well, because Queue uses heap sort which is not stable, so we
        # had to find a way to make it look stable ;-)
        with self.totalLock:
            self.total += 1
            self.put(( (-task.priority, time.time()), task ))


    def task_done(self):
        log.info('Task completed!')
        with self.totalLock:
            # if we finished all the tasks, reset the current total
            if self.empty():
                self.total = 0

            if self.progressCallback:
                self.progressCallback(self.total - self.qsize(), self.total)

        super(TaskManager, self).task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pygoo import Equal
from smewt.base import GraphAction, cachedmethod, utils, SmewtException, Media
from smewt.guessers.guesser import Guesser
from smewt.media.series import Episode, Series
from smewt.base.mediaobject import foundMetadata
from urllib import urlopen,  urlencode
from imdbmetadataprovider import IMDBMetadataProvider
import imdb
import logging

log = logging.getLogger('smewt.guessers.episodeimdb')


class EpisodeIMDB(GraphAction):

    def canHandle(self, query):
        if query.find_one(Media).type() not in [ 'video', 'subtitle' ]:
            raise SmewtException("%s: can only handle video or subtitle media objects" % self.__class__.__name__)

    def perform(self, query):
        self.checkValid(query)
        self.query = query

        log.debug('EpisodeImdb: finding more info on %s' % query.find_all(node_type = Episode))
        ep = query.find_one(Episode)

        if ep.get('series') is None:
            raise SmewtException("EpisodeIMDB: Episode doesn't contain 'series' field: %s" % ep)

        # little hack: if we have no season number, add 1 as default season number
        # (helps for series which have only 1 season)
        if ep.get('season') is None:
            ep.season = 1

        mdprovider = IMDBMetadataProvider()
        result = mdprovider.startEpisode(ep)

        # first, change the current series object by the one we found on the web
        oldseries = ep.series
        ep.series = query.add_object(result.find_one(Series))
        # and remove the stale series node
        query.delete_node(oldseries.node)

        # then add all the potential episodes
        for ep in result.find_all(Episode):
            query.add_object(ep, recurse = Equal.OnLiterals)

        return query


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2010 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import QSettings, QVariant
import smewt
from smewt.media import Series, Episode, Movie
from smewt.base import LocalCollection
from smewt.base.taskmanager import Task, TaskManager
from smewt.base.utils import smewtUserDirectory
from os.path import join, dirname, splitext
from smewt.taggers import EpisodeTagger, MovieTagger
import logging

log = logging.getLogger('smewt.base.smewtdaemon')


class SmewtDaemon(object):
    def __init__(self, progressCallback = None):
        super(SmewtDaemon, self).__init__()

        # FIXME: this is a lame hack to save the collection when we can...
        def callback(current, total):
            if total == 0:
                self.saveCollection()
            if progressCallback is not None:
                progressCallback(current, total)

        self.taskManager = TaskManager(progressCallback = callback)

        settings = QSettings()
        cfile = unicode(settings.value('collection_file').toString())
        if not cfile:
            cfile = join(smewtUserDirectory(), smewt.APP_NAME + '.collection')
            settings.setValue('collection_file', QVariant(cfile))

        csettings = unicode(settings.value('collection_settings').toString())
        if not csettings:
            csettings = join(smewtUserDirectory(), smewt.APP_NAME + '.collection_settings')
            settings.setValue('collection_settings', QVariant(csettings))


        self.collection = LocalCollection(taskManager = self.taskManager, settingsFile = csettings)
        #self.connect(self.collection, SIGNAL('updated'),
        #             self.refreshCollectionView)
        #self.connect(self.collection, SIGNAL('updated'),
        #             self.saveCollection)

        try:
            self.collection.load(cfile)
        except:
            log.warning('Could not load collection %s', cfile)

    def saveCollection(self):
        cfile = unicode(QSettings().value('collection_file').toString())
        self.collection.save(cfile)

    def shutdown(self):
        self.saveCollection()

    def quit(self):
        self.taskManager.abortAll()

    def updateCollection(self):
        self.collection.update()

    def rescanCollection(self):
        self.collection.rescan()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from base import SmewtDict, ValidatingSmewtDict, SmewtException, SmewtUrl, SolvingChain, cachedmethod, EventServer, utils, textutils
from base.mediaobject import Media, Metadata

import logging
logging.basicConfig(level = logging.INFO,
                    format = '%(levelname)-8s %(module)s:%(funcName)s -- %(message)s')

# we most likely never want this to be on debug mode, as it spits out way too much information
logging.getLogger('smewt.datamodel').setLevel(logging.INFO)
logging.getLogger('imdbpy').setLevel(logging.INFO)

# used to be able to store settings for different versions of Smewt installed on the same computer, ie: a stable
# and a development version
ORG_NAME = 'DigitalGaia'
APP_NAME = 'Smewt'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import SmewtException, SolvingChain, Media, utils
from smewt.media import Episode, Series, Subtitle
from smewt.taggers.tagger import Tagger
from smewt.guessers import *
from smewt.solvers import *
import logging

log = logging.getLogger('smewt.taggers.episodetagger')

class EpisodeTagger(Tagger):

    def perform(self, query):
        filenameMetadata = SolvingChain(EpisodeFilename(), MergeSolver(Episode)).solve(query)
        result = SolvingChain(EpisodeTVDB(), SimpleSolver(Episode)).solve(filenameMetadata)

        media = result.find_one(Media)

        # TODO: useless now?
        if not media.get('metadata'):
            log.warning('Could not find any tag for: %s' % media)

            # we didn't find any info outside of what the filename told us
            media.metadata = [ self.filenameMetadata ]

            # try anyway to get the correct series name and poster
            from smewt.guessers.tvdbmetadataprovider import TVDBMetadataProvider
            mdprovider = TVDBMetadataProvider()

            try:
                series = mdprovider.getSeries(media.metadata.series.title)
                result += media.metadata
                media.metadata.series = Series(title = series)
                media.metadata.episodeNumber = -1

                lores, hires = mdprovider.getPoster(series.movieID)
                media.metadata.series.loresImage = lores
                media.metadata.series.hiresImage = hires

            except SmewtException:
                log.warning('Could not even find a probable series name for: %s' % media)

        # import subtitles correctly
        if media.type() == 'subtitle':
            # FIXME: problem for vobsubs: as a media points to a single metadata object, we cannot
            # represent a .sub for 3 different languages...
            subs = []
            for language in utils.guessCountryCode(media.filename):
                subs += [ result.Subtitle(metadata = media.metadata,
                                          language = language) ]

                media.metadata = subs

        self.cleanup(result)
        #result.displayGraph()
        log.debug('Finished tagging: %s' % media)
        return result



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()

version = '0.2b1'

install_requires = [
    # List your project dependencies here.
    # For more details, see:
    # http://packages.python.org/distribute/setuptools.html#declaring-dependencies
    'pygoo',
    'pycurl',
    'cheetah',
    'lxml',
    'feedparser'
]

## media_imports = []
## for root, dirs, files in os.walk('smewt/media'):
##     media_imports += [ os.path.join(root, '*.' + ext)[6:] for ext in ('png', 'tmpl', 'css', 'html', 'js') ]

## icons_imports = [ 'icons/*.' + ext for ext in ('png', 'svg') ]

datafiles_exts = [ '*.png', '*.svg', '*.tmpl', '*.css', '*.html', '*.js' ]

setup(name = 'smewt',
      version = version,
      description = 'Smewt - a smart media manager.',
      long_description = README + '\n\n' + NEWS,
      classifiers = [], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords = 'pygoo object graph mapper',
      author = 'Nicolas Wack',
      author_email = 'wackou@gmail.com',
      url = 'http://www.smewt.com/',
      license = 'GPLv3',
      packages = find_packages(exclude = [ 'ez_setup', 'examples', 'tests' ]),
      #package_data = dict((package, datafiles_exts) for package in find_packages()),
      include_package_data = True,
      scripts = [ 'bin/smewg' ],
      install_requires = install_requires
      )
